# -*- coding: utf-8 -*-
from setuphelpers import *


##app_scheduled_tasks = [
##    "MicrosoftEdgeUpdateBrowserReplacementTask",
##    "MicrosoftEdgeUpdateTaskMachineCore",
##    "MicrosoftEdgeUpdateTaskMachineUA",
##    "MicrosoftEdgeUpdateTaskUser"
##]

app_services = [
    "MicrosoftEdgeElevationService",
    "edgeupdatem",
    "edgeupdate",
]

def install():
    # Declaring local variables
    bin_name = glob.glob("MicrosoftEdgeEnterprise*.msi")[0]
    app_uninstallkey = get_msi_properties(bin_name)["ProductCode"]

    # Disabling automatic updates (https://admx.help/?Category=EdgeChromium&Policy=Microsoft.Policies.Update::Pol_DefaultUpdatePolicy)
    registry_deletekey(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Microsoft", "EdgeUpdate")
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Microsoft\EdgeUpdate", "UpdateDefault", 0)
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Microsoft\EdgeUpdate", "DisableAutoUpdateChecksCheckboxValue", 1)
    registry_set(HKEY_LOCAL_MACHINE, r"SOFTWARE\Policies\Microsoft\EdgeUpdate", "AutoUpdateCheckPeriodMinutes", 0)

    stop_disable_edge_tasks()

    # Stopping application services
    for service_name in app_services:
        if service_installed(service_name):
            try:
                if service_is_running(service_name):
                    service_stop(service_name)
            except:
                print(f"Unable to stop the service: {service_name}")

    # for s in installed_softwares("Edge"):
    #     print("Found: %s (%s)" % (s["name"], s["version"]))
    #     print(s)

    if windows_version() < WindowsVersions.Windows10:
        # Uninstalling newer versions of the software for W7/8
        for to_uninstall in installed_softwares(name="^Microsoft Edge$"):
            if Version(to_uninstall["version"]) > Version(control.get_software_version()) or force:
                print("Removing: %s (%s)" % (to_uninstall["name"], to_uninstall["version"]))
                try:
                    killalltasks(control.get_impacted_process_list())
                    run(uninstall_cmd(to_uninstall["key"]))
                    wait_uninstallkey_absent(to_uninstall["key"])
                except Exception as e:
                    print(e)
                    print("Remove failed: %s (%s)\nContinuing..." % (to_uninstall["name"], to_uninstall["version"]))

    # Installing the package
    install_msi_if_needed(
        bin_name,
        min_version=control.get_software_version(),
        timeout=900,
        remove_old_version=True,
    )

    if windows_version() > WindowsVersions.Windows10:
        # Avoiding the usage by WAPT of the app built-in Uninstallstring for Windows 10+
        uninstallkey.remove(app_uninstallkey)

    # Removing desktop shortcut if allowed
    if params.get("remove_desktop_shortcut"):
        remove_desktop_shortcut(control.name)

    stop_disable_edge_tasks()

    # Changing default start mode of the application services
    for service_name in app_services:
        set_service_start_mode(service_name, "Manual")

def get_edge_tasks():
    return [
        task for task in get_all_scheduled_tasks(enable_only=True)
        if task.startswith("MicrosoftEdgeUpdate")
    ]

def stop_edge_tasks():
    for task in get_edge_tasks():
        try:
            run(rf'schtasks /end /tn "{task}"')
        except:
            print(f"Unable to stop the task_name: {task}")

def disable_edge_tasks():
    for task in get_edge_tasks():
        try:
            disable_task(task)
        except:
            print(f"Unable to disable the task_name: {task}")

def stop_disable_edge_tasks():
    stop_edge_tasks()
    disable_edge_tasks()